From dcd3e5746dc00f6a551f0d528b11e679eff3cd3f Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?S=C3=A9bastien=20Toque?= <xilasz@gmail.com>
Date: Wed, 5 Aug 2015 07:13:53 +0200
Subject: [PATCH] Add preference to enable writing on ext sdcard

---
 vlc-android/res/xml/preferences.xml                |  4 ++
 .../org/videolan/vlc/gui/PreferencesActivity.java  | 70 ++++++++++++++++++++++
 2 files changed, 74 insertions(+)

diff --git a/vlc-android/res/xml/preferences.xml b/vlc-android/res/xml/preferences.xml
index b92d6bf..f6b8b64 100644
--- a/vlc-android/res/xml/preferences.xml
+++ b/vlc-android/res/xml/preferences.xml
@@ -43,6 +43,10 @@
             android:key="screen_orientation"
             android:title="@string/screen_orientation" />
 
+        <PreferenceScreen
+            android:key="ext_sdcard_write"
+            android:title="Autoriser l'écriture sur la carte SD" />
+
     </PreferenceCategory>
     <PreferenceCategory android:title="@string/extra_prefs_category" >
         <PreferenceScreen
diff --git a/vlc-android/src/org/videolan/vlc/gui/PreferencesActivity.java b/vlc-android/src/org/videolan/vlc/gui/PreferencesActivity.java
index 1f02935..653ddfc 100644
--- a/vlc-android/src/org/videolan/vlc/gui/PreferencesActivity.java
+++ b/vlc-android/src/org/videolan/vlc/gui/PreferencesActivity.java
@@ -20,12 +20,16 @@
 
 package org.videolan.vlc.gui;
 
+import android.annotation.TargetApi;
 import android.app.Dialog;
+import android.content.ContentResolver;
 import android.content.DialogInterface;
 import android.content.Intent;
 import android.content.SharedPreferences;
 import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
+import android.content.UriPermission;
 import android.graphics.drawable.Drawable.ConstantState;
+import android.net.Uri;
 import android.os.Build;
 import android.os.Bundle;
 import android.preference.CheckBoxPreference;
@@ -38,8 +42,10 @@ import android.preference.PreferenceActivity;
 import android.preference.PreferenceGroup;
 import android.preference.PreferenceManager;
 import android.preference.PreferenceScreen;
+import android.support.v4.provider.DocumentFile;
 import android.support.v7.app.AlertDialog;
 import android.support.v7.widget.Toolbar;
+import android.util.Log;
 import android.util.TypedValue;
 import android.view.LayoutInflater;
 import android.view.View;
@@ -48,6 +54,7 @@ import android.view.Window;
 import android.widget.LinearLayout;
 import android.widget.ListView;
 
+import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.libvlc.util.HWDecoderUtil;
 import org.videolan.vlc.MediaDatabase;
 import org.videolan.vlc.PlaybackService;
@@ -60,6 +67,8 @@ import org.videolan.vlc.util.VLCInstance;
 import org.videolan.vlc.util.VLCOptions;
 import org.videolan.vlc.BuildConfig;
 
+import java.util.List;
+
 @SuppressWarnings("deprecation")
 public class PreferencesActivity extends PreferenceActivity implements OnSharedPreferenceChangeListener, PlaybackService.Client.Callback {
 
@@ -75,6 +84,7 @@ public class PreferencesActivity extends PreferenceActivity implements OnSharedP
     public final static String VIDEO_RESTORE = "video_restore";
     public final static int RESULT_RESCAN = RESULT_FIRST_USER + 1;
     public final static int RESULT_RESTART = RESULT_FIRST_USER + 2;
+    public final static int REQUEST_CODE_STORAGE_ACCES = 42;
 
     private PlaybackService.Client mClient = new PlaybackService.Client(this, this);
     private PlaybackService mService;
@@ -285,6 +295,25 @@ public class PreferencesActivity extends PreferenceActivity implements OnSharedP
             }
         });
 
+        // Directories
+        Preference extSdCardWritePref = findPreference("ext_sdcard_write");
+        if (AndroidUtil.isLolliPopOrLater()) {
+            extSdCardWritePref.setSummary(getUriPermissions());
+            extSdCardWritePref.setOnPreferenceClickListener(
+                    new OnPreferenceClickListener() {
+                        @TargetApi(Build.VERSION_CODES.LOLLIPOP)
+                        @Override
+                        public boolean onPreferenceClick(Preference preference) {
+                            Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT_TREE);
+                            startActivityForResult(intent, REQUEST_CODE_STORAGE_ACCES);
+                            return true;
+                        }
+                    });
+        }
+        else {
+            extSdCardWritePref.setEnabled(false);
+        }
+
         /*** SharedPreferences Listener to apply changes ***/
         SharedPreferences sharedPrefs = PreferenceManager.getDefaultSharedPreferences(this);
         sharedPrefs.registerOnSharedPreferenceChangeListener(this);
@@ -351,6 +380,47 @@ public class PreferencesActivity extends PreferenceActivity implements OnSharedP
         });
     }
 
+    @TargetApi(Build.VERSION_CODES.KITKAT)
+    private String getUriPermissions() {
+        StringBuilder sb = new StringBuilder();
+        List<UriPermission> persistedUriPermissions = getContentResolver().getPersistedUriPermissions();
+        for (UriPermission uriPermission : persistedUriPermissions) {
+            final DocumentFile file = DocumentFile.fromTreeUri(this, uriPermission.getUri());
+            sb.append(uriPermission.getUri().getPath() + "\n");
+        }
+        return sb.toString();
+    }
+
+    @TargetApi(Build.VERSION_CODES.KITKAT)
+    @Override
+    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
+        super.onActivityResult(requestCode, resultCode, data);
+        if (requestCode == REQUEST_CODE_STORAGE_ACCES && AndroidUtil.isLolliPopOrLater()) {
+            if (resultCode == RESULT_OK) {
+                Uri treeUri = data.getData();
+                final DocumentFile treeFile = DocumentFile.fromTreeUri(this, treeUri);
+                ContentResolver contentResolver = getContentResolver();
+
+                // revoke access if a permission already exists
+                final List<UriPermission> persistedUriPermissions = contentResolver.getPersistedUriPermissions();
+                for (UriPermission uriPermission : persistedUriPermissions) {
+                    final DocumentFile file = DocumentFile.fromTreeUri(this, uriPermission.getUri());
+                    if (treeFile.getName().equals(file.getName())) {
+                        Log.d(TAG, "Revoking permission to " + treeFile);
+                        contentResolver.releasePersistableUriPermission(uriPermission.getUri(), Intent.FLAG_GRANT_READ_URI_PERMISSION | Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
+                        findPreference("ext_sdcard_write").setSummary(getUriPermissions());
+                        return;
+                    }
+                }
+
+                // else set permission
+                Log.d(TAG, "Taking permission to " + treeUri);
+                contentResolver.takePersistableUriPermission(treeUri, Intent.FLAG_GRANT_READ_URI_PERMISSION | Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
+                findPreference("ext_sdcard_write").setSummary(getUriPermissions());
+            }
+        }
+    }
+
     @Override
     public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
         if(key.equalsIgnoreCase("hardware_acceleration")
-- 
2.1.4

